"""Reflex custom control_components.md components for Clerk."""
import typing
from typing import Union

import reflex as rx
from reflex import ImportVar
from reflex.utils.serializers import serializer


class Javascript(str):
    pass


@serializer
def serialize_javascript(obj: Javascript) -> str:
    return f"{{{obj}}}"


class ClerkLoaded(rx.Component):
    """ClerkLoaded component."""

    # The React library to wrap.
    library = "@clerk/clerk-react"

    # The React component tag.
    tag = "ClerkLoaded"


class ClerkLoading(rx.Component):
    """ClerkLoading component."""

    # The React library to wrap.
    library = "@clerk/clerk-react"

    # The React component tag.
    tag = "ClerkLoading"


class Protect(rx.Component):
    """Protect component."""

    # The React library to wrap.
    library = "@clerk/clerk-react"

    # The React component tag.
    tag = "Protect"

    condition: Javascript = None
    """
    Optional conditional logic that renders the children if it returns true.
    """

    fallback: typing.Union[rx.Component, str] = None
    """
    An optional snippet of JSX to show when a user doesn't have the role or permission 
    to access the protected content.
    """

    permission: str = None
    """
    Optional string corresponding to a Role's Permission in the format org:<resource>:<action>
    """

    role: str = None
    """
    Optional string corresponding to an Organization's Role in the format org:<role>
    """

    # Make sure imports for the fallback component are included in the dependencies.
    def add_imports(self) -> dict[str, Union[str, ImportVar, list[Union[str, ImportVar]]]]:
        if isinstance(self.fallback, rx.Component):
            return self.fallback._get_all_imports()
        return {}


class MultisessionAppSupport(rx.Component):
    """
    The <MultisessionAppSupport> provides a wrapper for your React application
    that guarantees a full rerendering cycle everytime the current session and
    user changes.
    """

    # The React library to wrap.
    library = "@clerk/clerk-react"

    # The React component tag.
    tag = "MultisessionAppSupport"


class RedirectToSignIn(rx.Component):
    """
    The <RedirectToSignIn /> component will navigate to the sign in URL which has
    been configured in your application instance. The behavior will be just like a
    server-side (3xx) redirect, and will override the current location in the history
    stack.
    """
    # The React library to wrap.
    library = "@clerk/clerk-react"

    # The React component tag.
    tag = "RedirectToSignIn"


class RedirectToSignUp(rx.Component):
    """
    The <RedirectToSignUp /> component will navigate to the sign in URL which has
    been configured in your application instance. The behavior will be just like a
    server-side (3xx) redirect, and will override the current location in the history
    stack.
    """
    # The React library to wrap.
    library = "@clerk/clerk-react"

    # The React component tag.
    tag = "RedirectToSignUp"


class RedirectToUserProfile(rx.Component):
    """
    The <RedirectToUserProfile /> component will navigate to the sign in URL which has
    been configured in your application instance. The behavior will be just like a
    server-side (3xx) redirect, and will override the current location in the history
    stack.
    """
    # The React library to wrap.
    library = "@clerk/clerk-react"

    # The React component tag.
    tag = "RedirectToUserProfile"


class RedirectToOrganizationProfile(rx.Component):
    """
    The <RedirectToOrganizationProfile /> component will navigate to the sign in URL which has
    been configured in your application instance. The behavior will be just like a
    server-side (3xx) redirect, and will override the current location in the history
    stack.
    """
    # The React library to wrap.
    library = "@clerk/clerk-react"

    # The React component tag.
    tag = "RedirectToOrganizationProfile"


class RedirectToCreateOrganization(rx.Component):
    """
    The <RedirectToCreateOrganization /> component will navigate to the sign in URL which has
    been configured in your application instance. The behavior will be just like a
    server-side (3xx) redirect, and will override the current location in the history
    stack.
    """
    # The React library to wrap.
    library = "@clerk/clerk-react"

    # The React component tag.
    tag = "RedirectToCreateOrganization"


class SignedIn(rx.Component):
    """
    The <SignedIn> component offers authentication checks as a cross-cutting concern. Any
    children components wrapped by a <SignedIn> component will be rendered only if there's
    a User with an active Session signed in your application.
    """
    # The React library to wrap.
    library = "@clerk/clerk-react"

    # The React component tag.
    tag = "SignedIn"


class SignedOut(rx.Component):
    """
    The <SignedOut> component offers authentication checks as a cross-cutting concern. Any
    children components wrapped by a <SignedOut> component will be rendered only if there's
    a User with an active Session signed in your application.
    """
    # The React library to wrap.
    library = "@clerk/clerk-react"

    # The React component tag.
    tag = "SignedOut"


def clerk_loaded(*children: rx.Component) -> ClerkLoaded:
    """
    The <ClerkLoaded> component indicates that Clerk is fully loaded.

    Args:
        *children: Zero or more child components that will be rendered inside the ClerkLoaded component.

    Returns:
        A ClerkLoaded component instance that can be rendered.
    """
    return ClerkLoaded.create(*children)


def clerk_loading(*children: rx.Component) -> ClerkLoading:
    """
    The <ClerkLoading> component indicates that Clerk is still loading.

    Args:
        *children: Zero or more child components that will be rendered inside the ClerkLoading component.

    Returns:
        A ClerkLoading component instance that can be rendered.
    """
    return ClerkLoading.create(*children)


def protect(
        *children: rx.Component,
        condition: Javascript = None,
        fallback: typing.Union[rx.Component, str] = None,
        permission: str = None,
        role: str = None
) -> Protect:
    """
    The <Protect> component conditionally renders its children based on the provided logic.

    Args:
        *children: Zero or more child components that will be rendered inside the Protect component.
        condition: Optional conditional logic that renders the children if it returns true.
        fallback: An optional snippet of JSX to show when a user doesn't have the role or permission to access the protected content.
        permission: Optional string corresponding to a Role's Permission in the format org:<resource>:<action>.
        role: Optional string corresponding to an Organization's Role in the format org:<role>.

    Returns:
        A Protect component instance that can be rendered.
    """
    return Protect.create(
        *children,
        condition=condition,
        fallback=fallback,
        permission=permission,
        role=role
    )


def multisession_app_support(*children: rx.Component) -> MultisessionAppSupport:
    """
    The <MultisessionAppSupport> provides a wrapper for your React application that guarantees a full rerendering cycle everytime the current session and user changes.

    Args:
        *children: Zero or more child components that will be rendered inside the MultisessionAppSupport component.

    Returns:
        A MultisessionAppSupport component instance that can be rendered.
    """
    return MultisessionAppSupport.create(*children)


def redirect_to_sign_in() -> RedirectToSignIn:
    """
    The <RedirectToSignIn /> component navigates to the sign-in URL configured in your application instance.

    Returns:
        A RedirectToSignIn component instance that can be rendered.
    """
    return RedirectToSignIn.create()


def redirect_to_sign_up() -> RedirectToSignUp:
    """
    The <RedirectToSignUp /> component navigates to the sign-up URL configured in your application instance.

    Returns:
        A RedirectToSignUp component instance that can be rendered.
    """
    return RedirectToSignUp.create()


def redirect_to_user_profile() -> RedirectToUserProfile:
    """
    The <RedirectToUserProfile /> component navigates to the user profile URL configured in your application instance.

    Returns:
        A RedirectToUserProfile component instance that can be rendered.
    """
    return RedirectToUserProfile.create()


def redirect_to_organization_profile() -> RedirectToOrganizationProfile:
    """
    The <RedirectToOrganizationProfile /> component navigates to the organization profile URL configured in your application instance.

    Returns:
        A RedirectToOrganizationProfile component instance that can be rendered.
    """
    return RedirectToOrganizationProfile.create()


def redirect_to_create_organization(*children: rx.Component) -> RedirectToCreateOrganization:
    """
    The <RedirectToCreateOrganization /> component navigates to the create organization URL configured in your application instance.

    Args:
        *children: Zero or more child components that will be rendered inside the RedirectToCreateOrganization component.

    Returns:
        A RedirectToCreateOrganization component instance that can be rendered.
    """
    return RedirectToCreateOrganization.create(*children)


def signed_in(*children: rx.Component) -> SignedIn:
    """
    The <SignedIn> component renders its children only if there's a User with an active Session signed in your application.

    Args:
        *children: Zero or more child components that will be rendered inside the SignedIn component.

    Returns:
        A SignedIn component instance that can be rendered.
    """
    return SignedIn.create(*children)


def signed_out(*children: rx.Component) -> SignedOut:
    """
    The <SignedOut> component renders its children only if there's no User with an active Session signed in your application.

    Args:
        *children: Zero or more child components that will be rendered inside the SignedOut component.

    Returns:
        A SignedOut component instance that can be rendered.
    """
    return SignedOut.create(*children)



from typing import Optional, List, Dict, Any, Literal

from reflex.base import pydantic


class ClerkError(pydantic.BaseModel):
    """
    Represents an error response from the Clerk API.
    """
    message: str
    code: Optional[int] = None


class DeletedObjectResponse(pydantic.BaseModel):
    """
    Represents the response for a successfully deleted object.
    """
    deleted: bool
    object: str


class PaginationMeta(pydantic.BaseModel):
    """
    Metadata for pagination.
    """
    total_count: int
    limit: int
    offset: int


class Verification(pydantic.BaseModel):
    """
    Represents the verification details of an email address or phone number.

    Attributes:
        strategy: The strategy pertaining to the parent sign-up or sign-in attempt.
        status: The state of the verification.
        attempts: The number of attempts related to the verification.
        expire_at: The time the verification will expire at.
    """
    strategy: Optional[
        Literal[
            "oauth_google", "oauth_mock", "admin",
            "phone_code", "email_code", "reset_password_email_code",
            "web3_metamask_signature"
        ]]
    status: Optional[Literal['unverified', 'verified', 'transferable', 'failed', 'expired']]
    nonce: Optional[str] = None
    attempts: Optional[int] = None
    expire_at: Optional[int] = None
    error: Optional[ClerkError] = None
    external_verification_redirect_url: Optional[str] = None


class IdentificationLink(pydantic.BaseModel):
    """
    Represents a link between an email address or phone number and another identification type.

    Attributes:
        type: One of "oauth_google", "oauth_mock", or "saml"
        id: A unique identifier for this link.
    """
    type: Literal["oauth_google", "oauth_mock", "saml"]
    id: str


class EmailAddress(pydantic.BaseModel):
    """
    Represents an email address associated with a user.

    Attributes:
        id: A unique identifier for this email address.
        email_address: The value of this email address.
        verification: An object holding information on the verification of this email address.
        linked_to: An array of objects containing information about any identifications that might be linked to this email address.
        created_at: Unix timestamp of creation
        updated_at: Unix timestamp of latest update
    """
    id: str
    object: Literal['email_address']
    email_address: str
    reserved: bool
    verification: Optional[Verification] = None
    linked_to: List[IdentificationLink]
    created_at: int
    updated_at: int


class EmailAddressResponse(pydantic.BaseModel):
    """
    Represents the response containing an email address.
    """
    data: EmailAddress


class PhoneNumber(pydantic.BaseModel):
    """
    Represents a phone number associated with a user.

    Attributes:
        id: A unique identifier for this phone number.
        phone_number: The value of this phone number, in [E.164 format](https://en.wikipedia.org/wiki/E.164).
        reserved_for_second_factor: Set to true if this phone number is reserved for multi-factor authentication (2FA). Set to false otherwise.
        default_second_factor: Marks this phone number as the default second factor for multi-factor authentication(2FA). A user can have exactly one default second factor.
        verification: An object holding information on the verification of this phone number.
        linked_to: An object containing information about any other identification that might be linked to this phone number.
        created_at: Unix timestamp of creation
        updated_at: Unix timestamp of latest update
    """
    id: str
    object: Literal['phone_number']
    phone_number: str
    reserved_for_second_factor: Optional[bool]
    default_second_factor: Optional[bool]
    reserved: bool
    verification: Optional[Verification] = None
    linked_to: List[IdentificationLink]
    backup_codes: Optional[List[str]] = None
    created_at: int
    updated_at: int


class PhoneNumberResponse(pydantic.BaseModel):
    """
    Represents the response containing a phone number.
    """
    data: PhoneNumber


class Session(pydantic.BaseModel):
    """
    Represents a session object.
    """
    id: str
    object: str
    status: str
    user_id: str
    client_id: str
    created_at: int
    updated_at: int
    last_active_at: Optional[int] = None


class SessionListResponse(pydantic.BaseModel):
    """
    Represents the response containing a list of sessions.
    """
    data: List[Session]
    meta: PaginationMeta


class SessionResponse(pydantic.BaseModel):
    """
    Represents the response containing a session.
    """
    data: Session


class Client(pydantic.BaseModel):
    """
    Represents a client object.
    """
    id: str
    object: str
    created_at: int
    updated_at: int
    last_sign_in_at: Optional[int] = None
    sign_in_attempt_id: Optional[str] = None
    session_ids: Optional[List[str]] = None
    sign_up_attempt_id: Optional[str] = None
    status: Optional[str] = None


class ClientListResponse(pydantic.BaseModel):
    """
    Represents the response containing a list of clients.
    """
    data: List[Client]
    meta: PaginationMeta


class ClientResponse(pydantic.BaseModel):
    """
    Represents the response containing a client.
    """
    data: Client


class Template(pydantic.BaseModel):
    """
    Represents a template object for email or SMS.
    """
    id: str
    object: str
    name: str
    subject: Optional[str] = None
    markup: Optional[str] = None
    body: str
    delivered_by_clerk: bool
    from_email_name: Optional[str] = None
    reply_to_email_name: Optional[str] = None


class TemplateListResponse(pydantic.BaseModel):
    """
    Represents the response containing a list of templates.
    """
    data: List[Template]
    meta: PaginationMeta


class TemplateResponse(pydantic.BaseModel):
    """
    Represents the response containing a template.
    """
    data: Template


class Web3Wallet(pydantic.BaseModel):
    """
    Represents a Web3 wallet address associated with a user.

    The address can be used as a proof of identification for users.

    Web3 addresses must be verified to ensure that they can be assigned
    to their rightful owners. The verification is completed via Web3 wallet
    browser extensions, such as [Metamask][1]. The Web3Wallet3 object holds all
    the necessary state around the verification process.

    [1]: https://metamask.io/

    Attributes:
        id: A unique identifier for this web3 wallet.
        web3_wallet: In [Ethereum](https://docs.metamask.io/guide/common-terms.html#address-public-key),
                     the address is made up of 0x + 40 hexadecimal characters.
        created_at: The date and time when the passkey was created.
        updated_at: The date and time when the passkey was updated.
    """
    id: str
    web3_wallet: str
    created_at: int
    updated_at: int
    verification: Optional[Verification] = None


class SAMLAccount(pydantic.BaseModel):
    """
    Represents a SAML account associated with a user.

    Attributes:
        id: A unique identifier for the SAML account.
        object: The type of object, typically 'saml_account'.
        provider: The SAML provider associated with the account.
        active: A boolean indicating whether the SAML account is active.
        email_address: The email address associated with the SAML account.
        first_name: The first name of the user associated with the SAML account.
        last_name: The last name of the user associated with the SAML account.
        provider_user_id: An optional identifier for the user within the SAML provider.
        public_metadata: Metadata for the SAML account.
        verification: An optional verification object associated with the SAML account.
    """
    id: str
    object: str
    provider: str
    active: bool
    email_address: str
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    provider_user_id: Optional[str] = None
    public_metadata: Dict[str, Any]
    verification: Optional[Verification] = None


class PasskeyResource(pydantic.BaseModel):
    """
    Represents a passkey associated with a user response.

    Attributes:
        id: The passkey's unique ID generated by Clerk.
        verification: Verification details for the passkey.
        name: The passkey's name.
        created_at: The date and time when the passkey was created.
        updated_at: The date and time when the passkey was updated.
        last_used_at: The date and time when the passkey was last used.

    """

    id: str
    """The passkey's unique ID generated by Clerk."""

    verification: Verification
    """Verification details for the passkey."""

    name: str
    """The passkey's name."""

    created_at: str
    """The date and time when the passkey was created."""

    updated_at: str
    """The date and time when the passkey was updated."""

    last_used_at: str
    """The date and time when the passkey was last used."""


class User(pydantic.BaseModel):
    """
    Represents a user object with various attributes related to their profile, authentication, and metadata.

    See the [Clerk documentation][1] for more details.

    [1]: https://clerk.com/docs/users/overview

    Attributes:
        id: A unique identifier for the user.
        object: The type of object, typically 'user'.
        external_id: An optional external identifier for the user.
        primary_email_address_id: The unique identifier for the EmailAddress that the user has set as primary.
        primary_phone_number_id: The unique identifier for the PhoneNumber that the user has set as primary.
        primary_web3_wallet_id: The unique identifier for the Web3Wallet that the user signed up with.
        username: The user's username.
        first_name: The user's first name.
        last_name: The user's last name.
        profile_image_url: Holds the default avatar or user's uploaded profile image. Compatible with Clerk's Image Optimization.
        image_url: The URL of the user's profile image.
        passkeys: An array of passkeys associated with the user's account.
        has_image: A boolean to check if the user has uploaded an image or one was copied from OAuth. Returns false if Clerk is displaying an avatar for the user.
        public_metadata: Metadata that can be read from the Frontend API and Backend API and can be set only from the Backend API.
        private_metadata: Metadata that can be read and set only from the Backend API.
        unsafe_metadata: Metadata that can be read and set from the Frontend API. Often used for custom fields attached to the User object.
        email_addresses: An array of all the EmailAddress objects associated with the user, including the primary.
        phone_numbers: An array of all the PhoneNumber objects associated with the user, including the primary.
        web3_wallets: An array of all the Web3Wallet objects associated with the user, including the primary.
        saml_accounts: An experimental list of SAML accounts associated with the user.
        password_enabled: A boolean indicating whether the user has a password on their account.
        two_factor_enabled: A boolean indicating whether the user has enabled two-factor authentication.
        totp_enabled: A boolean indicating whether the user has enabled TOTP by generating a TOTP secret and verifying it via an authenticator app.
        backup_code_enabled: A boolean indicating whether the user has enabled Backup codes.
        last_sign_in_at: The date when the user last signed in, may be empty if the user has never signed in.
        banned: A boolean indicating whether the user is banned.
        locked: A boolean indicating whether the user is locked.
    """

    id: str
    """A unique identifier for the user."""

    object: str
    """The type of object, typically 'user'."""

    external_id: Optional[str] = None
    """An optional external identifier for the user."""

    primary_email_address_id: Optional[str] = None
    """The unique identifier for the EmailAddress that the user has set as primary."""

    primary_phone_number_id: Optional[str] = None
    """The unique identifier for the PhoneNumber that the user has set as primary."""

    primary_web3_wallet_id: Optional[str] = None
    """The unique identifier for the Web3Wallet that the user signed up with."""

    username: Optional[str] = None
    """The user's username."""

    first_name: Optional[str] = None
    """The user's first name."""

    last_name: Optional[str] = None
    """The user's last name."""

    profile_image_url: Optional[str] = None
    """Holds the default avatar or user's uploaded profile image. Compatible with Clerk's Image Optimization."""

    image_url: Optional[str] = None
    """The URL of the user's profile image."""

    passkeys: Optional[List[PasskeyResource]] = None
    """An array of passkeys associated with the user's account."""

    has_image: bool
    """
    A boolean to check if the user has uploaded an image or one was copied from OAuth.
     Returns false if Clerk is displaying an avatar for the user.
     """

    public_metadata: Dict[str, Any]
    """Metadata that can be read from the Frontend API and Backend API and can be set only from the Backend API."""

    private_metadata: Optional[Dict[str, Any]] = None
    """Metadata that can be read and set only from the Backend API."""

    unsafe_metadata: Dict[str, Any]
    """
    Metadata that can be read and set from the Frontend API. Often used
    for custom fields attached to the User object.
    """

    email_addresses: List[EmailAddress]
    """An array of all the EmailAddress objects associated with the user, including the primary."""

    phone_numbers: List[PhoneNumber]
    """An array of all the PhoneNumber objects associated with the user, including the primary."""

    web3_wallets: List[Web3Wallet]
    """An array of all the Web3Wallet objects associated with the user, including the primary."""

    saml_accounts: List[SAMLAccount]
    """An experimental list of SAML accounts associated with the user."""

    password_enabled: bool
    """A boolean indicating whether the user has a password on their account."""

    two_factor_enabled: bool
    """A boolean indicating whether the user has enabled two-factor authentication."""

    totp_enabled: bool
    """
    A boolean indicating whether the user has enabled TOTP by generating a TOTP secret
    and verifying it via an authenticator app.
    """

    backup_code_enabled: bool
    """A boolean indicating whether the user has enabled Backup codes."""

    last_sign_in_at: Optional[int] = None
    """The date when the user last signed in, may be empty if the user has never signed in."""

    banned: bool
    """A boolean indicating whether the user is banned."""

    locked: bool
    """A boolean indicating whether the user is locked."""


class UserListResponse(pydantic.BaseModel):
    """
    Represents the response containing a list of users.
    """
    data: List[User]
    meta: PaginationMeta


class UserCountResponse(pydantic.BaseModel):
    """
    Represents the response containing the count of users.
    """
    count: int


class InterstitialResponse(pydantic.BaseModel):
    """
    Represents the response containing the interstitial HTML.
    """
    html: str


class Key(pydantic.BaseModel):
    """
    Represents a key in the JSON Web Key Set (JWKS).
    """
    kty: str
    use: str
    kid: str
    n: str
    e: str


class JWKSResponse(pydantic.BaseModel):
    """
    Represents the response containing the JSON Web Key Set (JWKS).
    """
    keys: List[Key]



from .clerk_client import clerk_client
from .clerk_client.clerk_response_models import User
from .lib.clerk_provider import ClerkState, clerk_provider
from .lib.control import signed_in, signed_out, redirect_to_sign_in, redirect_to_sign_up, redirect_to_user_profile, \
    redirect_to_organization_profile, redirect_to_create_organization, multisession_app_support, clerk_loading, \
    clerk_loaded, protect
from .lib.reflex_clerk import install_signin_page, install_signup_page, install_pages
from .lib.sign_in import sign_in, sign_in_button
from .lib.sign_out import sign_out_button
from .lib.sign_up import sign_up, sign_up_button
from .lib.user import user_button, user_profile



"""Reflex custom component SignIn."""
import typing

import reflex as rx
from reflex.utils.serializers import serializer

from .appearance import Appearance


class SignInInitialValues:
    email_address: str = None
    username: str = None
    phone_number: str = None


@serializer
def serialize_sign_in_initial_values(obj: SignInInitialValues) -> dict:
    return {
        "emailAddress": obj.email_address,
        "username": obj.username,
        "phoneNumber": obj.phone_number,
    }


class SignIn(rx.Component):
    """SignIn component."""

    # The React library to wrap.
    library = "@clerk/clerk-react"

    # The React component tag.
    tag = "SignIn"

    appearance: Appearance = None
    """
    The appearance of the component.
    """

    routing: typing.Literal['hash', 'path', 'virtual'] = None
    """The routing strategy for your pages.
    
    Defaults to 'path' in Next.js and Remix applications. Defaults to hash for all other SDK's.
    """

    path: str = None
    """
    The path where the component is mounted on when routing is set to path.
    It is ignored in hash- and virtual-based routing.

    For example: /sign-in.
    """

    sign_up_url: str = None
    """
    Full URL or path to the sign up page. Use this property to provide the target of the 
    'Sign Up' link that's rendered. It's recommended to use the environment variable instead.
    """

    force_redirect_url: str = None
    """
    If provided, this URL will always be redirected to after the user signs in. Takes priority
    over deprecated props such as after_sign_in_url and redirect_url. It's recommended to use
    the environment variable instead.
    """

    fallback_redirect_url: str = None
    """
    The fallback URL to redirect to after the user signs in, if there's no redirect_url in the
    path already. Defaults to /. Takes priority over deprecated props such as after_sign_in_url
    and redirect_url. It's recommended to use the environment variable instead.
    """

    sign_up_force_redirect_url: str = None
    """
    If provided, this URL will always be redirected to after the user signs up. Takes priority
    over deprecated props such as afterSignInUrl and redirectUrl. It's recommended to use the
    environment variable instead.
    """

    sign_up_fallback_redirect_url: str = None
    """
    The fallback URL to redirect to after the user signs up, if there's no redirect_url in the 
    path already. Defaults to /. Takes priority over deprecated props such as after_sign_in_url
    and redirect_url. It's recommended to use the environment variable instead.
    """

    initial_values: typing.Optional[SignInInitialValues] = None
    """The values used to prefill the sign-in fields with."""


class SignInButton(rx.Component):
    """SignInButton component."""

    # The React library to wrap.
    library = "@clerk/clerk-react"

    # The React component tag.
    tag = "SignInButton"

    force_redirect_url: str = None
    """
    If provided, this URL will always be redirected to after the user signs in. Takes priority
    over deprecated props such as after_sign_in_url and redirect_url. It's recommended to use
    the environment variable instead.
    """

    fallback_redirect_url: str = None
    """
    The fallback URL to redirect to after the user signs in, if there's no redirect_url in the
    path already. Defaults to /. Takes priority over deprecated props such as after_sign_in_url
    and redirect_url. It's recommended to use the environment variable instead.
    """

    sign_up_force_redirect_url: str = None
    """
    If provided, this URL will always be redirected to after the user signs up. Takes priority
    over deprecated props such as afterSignInUrl and redirectUrl. It's recommended to use the
    environment variable instead.
    """

    sign_up_fallback_redirect_url: str = None
    """
    The fallback URL to redirect to after the user signs up, if there's no redirect_url in the 
    path already. Defaults to /. Takes priority over deprecated props such as after_sign_in_url
    and redirect_url. It's recommended to use the environment variable instead.
    """

    mode: typing.Literal['redirect', 'modal'] = None
    """
    Determines what happens when a user clicks on the <SignInButton>. Setting this to 'redirect'
    will redirect the user to the sign-in route. Setting this to 'modal' will open a modal on 
    the current route.
    
    Defaults to 'redirect'.
    """


# sign_in_button = SignInButton.create
def sign_in_button(
        *children: rx.Component,
        force_redirect_url: typing.Optional[str] = None,
        fallback_redirect_url: typing.Optional[str] = None,
        sign_up_force_redirect_url: typing.Optional[str] = None,
        sign_up_fallback_redirect_url: typing.Optional[str] = None,
        mode: typing.Literal['redirect', 'modal'] = 'redirect'
) -> SignInButton:
    """
    The <SignInButton> component is a button that links to the sign-in page or displays the sign-in modal.

    This button is un-styled, and is intended to wrap around an rx.button component.

    Examples:
        ```python
        import reflex_clerk as clerk

        def page():
            clerk.clerk_provider(
                rx.box(
                    clerk.sign_in_button(
                        rx.button("Sign In")
                    )
                ),
                publishable_key="pk_my_publishable_key"
            )
        ```

    Args:
        *children: Zero or more child components that will be rendered inside the sign-in button.
        force_redirect_url: The URL to redirect to after the user successfully signs in. Defaults to None.
        fallback_redirect_url: The URL to redirect to if an error occurs during the sign-in process. Defaults to None.
        sign_up_force_redirect_url: The URL to redirect to after the user successfully signs up. Defaults to None.
        sign_up_fallback_redirect_url: The URL to redirect to if an error occurs during the sign-up process. Defaults to None.
        mode: The mode of the sign-in button. It can be either 'redirect' or 'modal'. Defaults to None.

    Returns:
        A SignInButton component instance that can be rendered.
    """
    return SignInButton.create(
        *children,
        force_redirect_url=force_redirect_url,
        fallback_redirect_url=fallback_redirect_url,
        sign_up_force_redirect_url=sign_up_force_redirect_url,
        sign_up_fallback_redirect_url=sign_up_fallback_redirect_url,
        mode=mode
    )


# sign_in = SignIn.create
def sign_in(
        *children: rx.Component,
        appearance: Appearance = None,
        routing: typing.Literal['hash', 'path', 'virtual'] = None,
        path: str = None,
        sign_up_url: str = None,
        force_redirect_url: str = None,
        fallback_redirect_url: str = None,
        sign_up_force_redirect_url: str = None,
        sign_up_fallback_redirect_url: str = None,
        initial_values: typing.Optional[SignInInitialValues] = None
) -> SignIn:
    """
    The <SignIn> component renders a sign-in form.

    Instead of using this component directly, consider using the pre-configured sign-in
    page configured with [`reflex_clerk.install_signin_page`][].

    This form can be styled and configured with various routing and redirection options.

    Examples:
        ```python
        import reflex_clerk as clerk

        def page():
            clerk.clerk_provider(
                rx.box(
                    rx.heading("Welcome, please sign in!"),
                    clerk.sign_in()
                ),
                publishable_key="pk_my_publishable_key"
            )
        ```

    Args:
        appearance: The appearance configuration for the sign-in form.
        routing: The routing strategy for the pages. Can be 'hash', 'path', or 'virtual'. Defaults to None.
        path: The path where the component is mounted when routing is set to 'path'. Ignored for 'hash' and 'virtual' routing.
        sign_up_url: Full URL or path to the sign-up page.
        force_redirect_url: URL to redirect to after sign-in. Takes priority over deprecated props.
        fallback_redirect_url: Fallback URL to redirect to after sign-in if no redirect URL is provided in the path. Defaults to '/'.
        sign_up_force_redirect_url: URL to redirect to after sign-up. Takes priority over deprecated props.
        sign_up_fallback_redirect_url: Fallback URL to redirect to after sign-up if no redirect URL is provided in the path. Defaults to '/'.
        initial_values: Prefilled values for the sign-in fields. Defaults to None.

    Returns:
        A SignIn component instance that can be rendered.
    """
    return SignIn.create(
        *children,
        appearance=appearance,
        routing=routing,
        path=path,
        sign_up_url=sign_up_url,
        force_redirect_url=force_redirect_url,
        fallback_redirect_url=fallback_redirect_url,
        sign_up_force_redirect_url=sign_up_force_redirect_url,
        sign_up_fallback_redirect_url=sign_up_fallback_redirect_url,
        initial_values=initial_values
    )



import reflex as rx

from .clerk_provider import clerk_provider
from .sign_in import sign_in
from .sign_up import sign_up


def install_signin_page(app: rx.App, publishable_key=None, route="/signin", **props) -> None:
    """
    This method installs a login page on the given App instance.
    It creates a centered layout using the rx.center() function,
    with a sign-in form generated by the [reflex_clerk.sign_in][] function.
    The sign-in form is vertically stacked using the rx.vstack() function,
    with a spacing of 7 units between each element. The entire layout is
    set to a height of 100vh.

    The login page is then installed on the App instance using the
    clerk_provider() function. If a publishable key is provided,
    it will be used for authentication.

    Example:
        ```python
        import reflex_clerk
        app = App()
        reflex_clerk.install_signin_page(app, publishable_key='your_publishable_key')
        ```

    Parameters:
        app (rx.App): The app instance to install the login page on.
        publishable_key (str): The publishable key to use for authentication.
        route (str): The route to install the login page on.  Defaults to "/signin".


    """

    assert route.startswith("/"), "Expected route to be absolute (e.g. starts with '/')"

    signin_page = clerk_provider(
        rx.center(
            rx.vstack(
                sign_in(
                    path=route,
                    **props
                ),
                align="center",
                spacing="7",
            ),
            height="100vh",
        ),
        publishable_key=publishable_key
    )

    app.pages[route[1:] + "/[[...signin]]/index"] = signin_page


def install_signup_page(app: rx.App, publishable_key=None, route="/signup", **props) -> None:
    """
    This method installs a signup page on the given App instance.
    It creates a centered layout using the rx.center() function,
    with a sign-in form generated by the [reflex_clerk.sign_up][] function.
    The sign-in form is vertically stacked using the rx.vstack() function,
    with a spacing of 7 units between each element. The entire layout is
    set to a height of 100vh.

    The login page is then installed on the App instance using the
    clerk_provider() function. If a publishable key is provided,
    it will be used for authentication.

    Example:
        ```python
        import reflex_clerk
        app = App()
        reflex_clerk.install_signup_page(app, publishable_key='your_publishable_key')
        ```

    Parameters:
        app (rx.App): The app instance to install the login page on.
        publishable_key (str): The publishable key to use for authentication.
        route (str): The route to install the login page on.  Defaults to "/signup".


    """

    assert route.startswith("/"), "Expected route to be absolute (e.g. starts with '/')"

    signup_page = clerk_provider(
        rx.center(
            rx.vstack(
                sign_up(
                    path=route,
                    **props
                ),
                align="center",
                spacing="7",
            ),
            height="100vh",
        ),
        publishable_key=publishable_key
    )

    app.pages[route[1:] + "/[[...signup]]/index"] = signup_page


def install_pages(
        app: rx.App,
        publishable_key=None,
        signin_route="/signin",
        signup_route="/signup",
        **props
) -> None:
    """Installs the signin and signup pages for the given app.

    Args:
        app (rx.App): The app instance for which the pages need to be installed.
        publishable_key (str, optional): The publishable key to be used for payment processing. Defaults to None.
        signin_route (str, optional): The route at which the signin page should be installed. Defaults to "/signin".
        signup_route (str, optional): The route at which the signup page should be installed. Defaults to "/signup".

    Example:
        ```python
        import reflex_clerk as clerk
        import reflex as rx
        app = rx.App()
        clerk.install_pages(app, publishable_key='your_publishable_key')
        ```

    Returns:
        None
    """
    install_signin_page(app, publishable_key=publishable_key, route=signin_route, **props)
    install_signup_page(app, publishable_key=publishable_key, route=signup_route, **props)



"""Reflex custom component SignIn."""

import reflex as rx
from reflex.utils.serializers import serializer


class SignOutOptions:
    session_id: str = None
    """
    The ID of a specific session to sign out of. Useful for multi-session applications.
    """
    redirect_url: str = None
    """
    The redirect URL to navigate to after sign out is complete.
    """


@serializer
def serialize_sign_out_options(obj: SignOutOptions) -> dict:
    return {
        "sessionId": obj.session_id,
        "redirectUrl": obj.redirect_url,
    }


class SignOutButton(rx.Component):
    """SignOutButton component."""

    # The React library to wrap.
    library = "@clerk/clerk-react"

    # The React component tag.
    tag = "SignOutButton"

    options: SignOutOptions = None
    """Options for the sign out button."""

    redirect_url: str = None
    """
    The redirect URL to navigate to after sign out is complete.
    """


# sign_out_button = SignOutButton.create
def sign_out_button(
        *children: rx.Component,
        options: SignOutOptions = None,
        redirect_url: str = None
) -> SignOutButton:
    """
    The <SignOutButton> component is a button that signs out the user and redirects them to a specified URL.

    This button is un-styled, and is intended to wrap around an rx.button component.

    Examples:
        ```python
        import reflex_clerk as clerk

        def page():
            clerk.clerk_provider(
                rx.box(
                    clerk.sign_out_button(
                        rx.button("Sign Out")
                    )
                ),
                publishable_key="pk_my_publishable_key"
            )
        ```

    Args:
        *children: Zero or more child components that will be rendered inside the sign-out button.
        options: Options for the sign-out button. Defaults to None.
        redirect_url: The URL to redirect to after sign-out. Defaults to None.

    Returns:
        A SignOutButton component instance that can be rendered.
    """
    return SignOutButton.create(
        *children,
        options=options,
        redirect_url=redirect_url
    )



"""Reflex custom component SignIn."""
import typing

import reflex as rx

from .appearance import Appearance


class UserButton(rx.Component):
    """UserButton component."""

    # The React library to wrap.
    library = "@clerk/clerk-react"

    # The React component tag.
    tag = "UserButton"

    appearance: Appearance = None
    """
    Optional object to style your components. Will only affect Clerk Components 
    and not Account Portal pages.
    """

    show_name: bool = None
    """
    Controls if the user name is displayed next to the user image button.
    """

    sign_in_url: str = None
    """
    The full URL or path to navigate to when the Add another account button is clicked.
    It's recommended to use the environment variable instead.
    """

    user_profile_mode: typing.Literal['modal', 'navigation'] = None
    """
    Controls whether clicking the Manage your account button will cause the 
    <UserProfile /> component to open as a modal, or if the browser will navigate 
    to the userProfileUrl where <UserProfile /> is mounted as a page. Defaults to: 'modal'.
    """

    user_profile_url: str = None
    """
    The full URL or path leading to the user management interface.
    """

    after_sign_out_url: str = None
    """
    The full URL or path to navigate to after signing out from all accounts 
    (applies to both single-session and multi-session apps). Default is /.
    """

    after_multi_session_single_sign_out_url: str = None
    """
    The full URL or path to navigate to after signing out from currently active account 
    (multi-session apps).
    """

    after_switch_session_url: str = None
    """
    The full URL or path to navigate to after a successful account change (multi-session apps).
    """

    default_open: bool = None
    """
    Controls whether the <UserButton /> should open by default during the first render.
    """

    user_profile_props: dict = None
    """
    Specify options for the underlying <UserProfile /> component. 
    For example: {additionalOAuthScopes: {google: ['foo', 'bar'], github: ['qux']}}.
    """


class UserProfile(rx.Component):
    """UserProfile component."""

    # The React library to wrap.
    library = "@clerk/clerk-react"

    # The React component tag.
    tag = "UserProfile"

    appearance: Appearance = None
    """
    Optional object to style your components. Will only affect Clerk Components 
    and not Account Portal pages.
    """

    additional_oauth_scopes: dict = None
    """
    Specify additional scopes per OAuth provider that your users would like to
    provide if not already approved.
    
    For example: {"google": ['foo', 'bar'], "github": ['qux']}.
    """


# user_button = UserButton.create
def user_button(
        *children: rx.Component,
        appearance: Appearance = None,
        show_name: bool = None,
        sign_in_url: str = None,
        user_profile_mode: typing.Literal['modal', 'navigation'] = 'modal',
        user_profile_url: str = None,
        after_sign_out_url: str = None,
        after_multi_session_single_sign_out_url: str = None,
        after_switch_session_url: str = None,
        default_open: bool = None,
        user_profile_props: dict = None
) -> UserButton:
    """
    The <UserButton> component displays a user image button that can be configured with various options.

    Examples:
        ```python
        import reflex_clerk as clerk

        def page():
            clerk.clerk_provider(
                rx.box(
                    clerk.user_button()
                ),
                publishable_key="pk_my_publishable_key"
            )
        ```

    Args:
        appearance: Optional object to style the component. Will only affect Clerk Components and not Account Portal pages.
        show_name: Controls if the user name is displayed next to the user image button. Defaults to False.
        sign_in_url: Full URL or path to navigate to when the Add another account button is clicked.
        user_profile_mode: Controls whether clicking the Manage your account button will open the <UserProfile /> component as a modal or navigate to the userProfileUrl. Defaults to 'modal'.
        user_profile_url: Full URL or path leading to the user management interface.
        after_sign_out_url: Full URL or path to navigate to after signing out from all accounts. Defaults to '/'.
        after_multi_session_single_sign_out_url: Full URL or path to navigate to after signing out from the currently active account in multi-session apps.
        after_switch_session_url: Full URL or path to navigate to after a successful account change in multi-session apps.
        default_open: Controls whether the <UserButton /> should open by default during the first render. Defaults to None.
        user_profile_props: Options for the underlying <UserProfile /> component.

    Returns:
        A UserButton component instance that can be rendered.
    """
    return UserButton.create(
        *children,
        appearance=appearance,
        show_name=show_name,
        sign_in_url=sign_in_url,
        user_profile_mode=user_profile_mode,
        user_profile_url=user_profile_url,
        after_sign_out_url=after_sign_out_url,
        after_multi_session_single_sign_out_url=after_multi_session_single_sign_out_url,
        after_switch_session_url=after_switch_session_url,
        default_open=default_open,
        user_profile_props=user_profile_props
    )


# user_profile = UserProfile.create
def user_profile(
        *children: rx.Component,
        appearance: Appearance = None,
        additional_oauth_scopes: dict = None
) -> UserProfile:
    """
    The <UserProfile> component displays the user profile interface with various configuration options.

    Examples:
        ```python
        import reflex_clerk as clerk

        def page():
            clerk.clerk_provider(
                rx.box(
                    clerk.user_profile()
                ),
                publishable_key="pk_my_publishable_key"
            )
        ```

    Args:
        *children: Zero or more child components that will be rendered inside the user profile component.
        appearance: Optional object to style the component. Will only affect Clerk Components and not Account Portal pages.
        additional_oauth_scopes: Specify additional scopes per OAuth provider that your users would like to provide if not already approved. Defaults to None.

    Returns:
        A UserProfile component instance that can be rendered.
    """
    return UserProfile.create(
        *children,
        appearance=appearance,
        additional_oauth_scopes=additional_oauth_scopes
    )



import typing

from reflex.base import pydantic


class AppearanceVariables(pydantic.BaseModel):
    """Variables for the appearance."""

    colorPrimary: str = None
    # todo: flush this out


class Appearance(pydantic.BaseModel):
    """Appearance configuration for the ClerkProvider component."""

    baseTheme: typing.Union[typing.Literal["default", "dark", "shadesOfPurple", "neobrutalism"], typing.List[
        typing.Literal["default", "dark", "shadesOfPurple", "neobrutalism"]]] = None

    signIn: "Appearance" = None
    signUp: "Appearance" = None
    variables: AppearanceVariables = None
    elements: typing.Dict[str, typing.Any] = None



from typing import Optional, List

from reflex.base import pydantic


class VerifyClientRequest(pydantic.BaseModel):
    token: str


class CreateEmailAddressRequest(pydantic.BaseModel):
    user_id: str
    email_address: str
    verified: Optional[bool] = None
    primary: Optional[bool] = None


class UpdateEmailAddressRequest(pydantic.BaseModel):
    verified: Optional[bool] = None
    primary: Optional[bool] = None


class CreatePhoneNumberRequest(pydantic.BaseModel):
    user_id: str
    phone_number: str
    verified: Optional[bool] = None
    primary: Optional[bool] = None
    reserved_for_second_factor: Optional[bool] = None


class UpdatePhoneNumberRequest(pydantic.BaseModel):
    verified: Optional[bool] = None
    primary: Optional[bool] = None
    reserved_for_second_factor: Optional[bool] = None


class CreateUserRequest(pydantic.BaseModel):
    external_id: Optional[str] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    email_address: Optional[List[str]] = None
    phone_number: Optional[List[str]] = None
    web3_wallet: Optional[List[str]] = None
    username: Optional[str] = None
    password: Optional[str] = None
    password_digest: Optional[str] = None
    password_hasher: Optional[str] = None
    skip_password_checks: Optional[bool] = None
    skip_password_requirement: Optional[bool] = None
    totp_secret: Optional[str] = None
    backup_codes: Optional[List[str]] = None
    public_metadata: Optional[dict] = None
    private_metadata: Optional[dict] = None
    unsafe_metadata: Optional[dict] = None
    created_at: Optional[str] = None


class UpdateUserRequest(pydantic.BaseModel):
    external_id: Optional[str] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    primary_email_address_id: Optional[str] = None
    notify_primary_email_address_changed: Optional[bool] = None
    primary_phone_number_id: Optional[str] = None
    primary_web3_wallet_id: Optional[str] = None
    username: Optional[str] = None
    profile_image_id: Optional[str] = None
    password: Optional[str] = None
    password_digest: Optional[str] = None
    password_hasher: Optional[str] = None


class UpsertTemplateRequest(pydantic.BaseModel):
    name: str
    subject: Optional[str] = None
    markup: Optional[str] = None
    body: str
    delivered_by_clerk: Optional[bool] = None
    from_email_name: Optional[str] = None
    reply_to_email_name: Optional[str] = None


class PreviewTemplateRequest(pydantic.BaseModel):
    subject: Optional[str] = None
    body: str
    from_email_name: Optional[str] = None
    reply_to_email_name: Optional[str] = None


class ToggleTemplateDeliveryRequest(pydantic.BaseModel):
    delivered_by_clerk: Optional[bool] = None


class VerifySessionRequest(pydantic.BaseModel):
    token: str


class CreateSessionTokenFromTemplateRequest(pydantic.BaseModel):
    pass


class GetPublicInterstitialParams(pydantic.BaseModel):
    frontendApi: Optional[str] = None
    publishable_key: Optional[str] = None


class GetClientListParams(pydantic.BaseModel):
    limit: Optional[int] = None
    offset: Optional[int] = None


class GetSessionListParams(pydantic.BaseModel):
    client_id: Optional[str] = None
    user_id: Optional[str] = None
    status: Optional[str] = None
    limit: Optional[int] = None
    offset: Optional[int] = None


class GetUserListParams(pydantic.BaseModel):
    email_address: Optional[List[str]] = None
    phone_number: Optional[List[str]] = None
    external_id: Optional[List[str]] = None
    username: Optional[List[str]] = None
    web3_wallet: Optional[List[str]] = None
    user_id: Optional[List[str]] = None
    organization_id: Optional[List[str]] = None
    query: Optional[str] = None
    last_active_at_since: Optional[int] = None
    limit: Optional[int] = None
    offset: Optional[int] = None
    order_by: Optional[str] = None


class GetUsersCountParams(pydantic.BaseModel):
    email_address: Optional[List[str]] = None
    phone_number: Optional[List[str]] = None
    external_id: Optional[List[str]] = None
    username: Optional[List[str]] = None
    web3_wallet: Optional[List[str]] = None
    user_id: Optional[List[str]] = None
    query: Optional[str] = None



import json
import logging
import os
import typing
from typing import List, Union

import reflex as rx
from authlib.jose import jwt, JoseError, JWTClaims
from reflex import Component, ImportVar
from reflex.utils.serializers import serializer

from reflex_clerk.clerk_client import clerk_client, clerk_response_models
from reflex_clerk.clerk_client.clerk_client import ClerkAPIClient


@serializer
def serialize_exception(e: Exception) -> dict:
    return {"error": str(e)}


@serializer
def serialize_clerk_user(user: clerk_response_models.User) -> dict:
    return user.dict()


class ClerkState(rx.State):
    """
    A Reflex state object representing the current authenticated session and user;
    use this state to render details about the currently logged in user.

    Example:
        ```python
        import reflex_clerk as clerk

        clerk.signed_in(
            rx.cond(
                ClerkState.user.has_image,
                rx.chakra.avatar(
                    src=ClerkState.user.image_url,
                    name=ClerkState.user.first_name,
                    size="xl",
                ),
            )
        )
        ```

    Example:
        ```python
        import reflex_clerk as clerk

        clerk.protect(
            rx.fragment("You are logged in as ", clerk.ClerkState.user.first_name),
            fallback=rx.text("You need to sign in first!"),
        )
        ```


    """

    is_signed_in: bool = False
    """
    True if the user is signed in, False otherwise.
    
    This field is only set to true after server-side validation of the session is complete.
    Check if this is true before returning sensitive user state from other states.
    
    Example:
        ```python
        rx.cond(
            clerk.ClerkState.is_signed_in,
            rx.text("You are signed in"),
            rx.text("You are signed out")
        )
        ```
        
        _Consider using [reflex_clerk.signed_in][] and [reflex_clerk.signed_out][] instead
        to render different information based on if the user is signed in or not on the client
        side instead of on the server._

    """

    auth_error: typing.Optional[Exception] = None
    """
    Non-None if the user authentication process failed.
    
    This should be quite rare, and likely indicates an expired session.
    """

    claims: typing.Optional[JWTClaims] = None
    """
    This field contains the decoded JWT Session claims once server-side validation
    of the session is complete.  You can use
    [Clerk JWT Templates](https://clerk.com/docs/backend-requests/making/jwt-templates)
    to return additional information about the session to this field.
    """

    user_id: typing.Optional[str] = None
    """
    A variable representing the Clerk User ID of the currently logged in user.
    
    This field is only set to true after server-side validation of the session is complete.
    """

    user: typing.Optional[clerk_response_models.User] = None
    """
    A variable representing the Clerk User of the currently logged in user.

    This field is only set to true after server-side validation of the session is complete,
    and can be disabled for performance reasons using [set_fetch_user_on_auth][reflex_clerk.ClerkState.set_fetch_user_on_auth].
    """

    # static class variables
    _jwt_public_keys: List[typing.Dict[str, str]] = []
    _secret_key: str = None
    _clerk_api_client: ClerkAPIClient = None
    _fetch_user: bool = True

    # noinspection PyPropertyDefinition
    @classmethod
    @property
    def secret_key(cls) -> str:
        if cls._secret_key is None:
            if 'CLERK_SECRET_KEY' in os.environ:
                cls._secret_key = os.environ['CLERK_SECRET_KEY']

        return cls._secret_key

    # noinspection PyPropertyDefinition
    @classmethod
    @property
    def jwt_public_keys(cls) -> typing.List[typing.Dict[str, str]]:
        if not cls._jwt_public_keys:
            if 'CLERK_JWT_PUBLIC_KEYS' in os.environ:
                cls._jwt_public_keys = list(map(json.loads, os.environ['CLERK_JWT_PUBLIC_KEYS'].split(',')))
            if cls.secret_key and cls.clerk_api_client:
                cls._jwt_public_keys = cls._clerk_api_client.get_jwks().dict()['keys']
        return cls._jwt_public_keys

    # noinspection PyPropertyDefinition
    @classmethod
    @property
    def clerk_api_client(cls) -> clerk_client.ClerkAPIClient:
        if cls._clerk_api_client is None:
            cls._clerk_api_client = clerk_client.get_client(cls.secret_key)
        return cls._clerk_api_client

    @classmethod
    def set_fetch_user_on_auth(cls, fetch_user: bool):
        """
        This method is used to control whether the authenticating user should be
        fetched from Clerk upon successful authentication.

        Defaults to true; set this to False if the Clark User object is not being
        used, as it saves a backend call to Clark every time a user authenticates.

        Note:
            Setting this to True only fetches ClerkState.user once at the time of
            user authentication. This implementation doesn't automatically detect
             and update any changes in the underlying user object data, such as
            email address modifications. Therefore, to ensure you have the latest
            user data while logged in, especially if changes have been made, you
            should use the ClerkState.fetch_user reflex event to manually refresh
            the user data.

        Args:
            fetch_user (bool): Indicates whether the user should be fetched.
        """
        cls._fetch_user = fetch_user

    def set_clerk_session(self, token: str) -> None:
        """
        Validates a Clerk session token and optionally fetches the associated
        user object.  This is used internally by reflex_clerk to manage the
        current auth state of users: it is called by the frontend whenever
        the Clerk isSignedIn auth state changes from false to true.

        Args:
            token: A JWT token used to authenticate and authorize the user.
        """
        if not ClerkState._jwt_public_keys:
            print(f"No Clerk JWT public keys found. Skipping Clerk session set.")
            return

        try:
            decoded: JWTClaims = jwt.decode(token, {"keys": ClerkState.jwt_public_keys})
            self.is_signed_in = True
            self.claims = decoded
            self.user_id = decoded.get('sub')

            if self._fetch_user:
                self.fetch_user()

        except JoseError as e:
            self.auth_error = e
            logging.warning(f"Auth error: {e}")

    def clear_clerk_session(self):
        """
        Clears the clerk session by setting the sign-in status to False,
        resetting the claims to None, and clearing any authentication errors.

        This is used internally by reflex_clerk to manage the
        current auth state of users: it is called by the frontend whenever
        the Clerk isSignedIn auth state changes from true to false.


        :param self: The current instance of the class.
        """
        self.is_signed_in = False
        self.claims = None
        self.auth_error = None

    def clear_clerk_auth_error(self):
        """
        Clears the clerk authentication error.

        This method sets the `auth_error` attribute of the current instance to `None`.
        """
        self.auth_error = None

    def fetch_user(self):
        """
        Fetches ClerkState.user from the clerk backend API, using
        ClerkState.user_id.  Use this Reflex event if you want to force
        update the clerk user data after a user logs in (such as after modifying
        their account profile).
        """
        if self.user_id:
            user = self.clerk_api_client.get_user(self.user_id)
            self.set_user(user)


class ClerkSessionSynchronizer(rx.Component):
    """ClerkSessionSynchronizer component."""
    tag = "ClerkSessionSynchronizer"

    def add_imports(self) -> dict[str, Union[str, ImportVar, list[Union[str, ImportVar]]]]:
        addl_imports = {
            "@clerk/clerk-react": ["useAuth"],
            "react": ["useContext", "useEffect"],
            "/utils/context": ["EventLoopContext"],
            "/utils/state": ["Event"]
        }
        return addl_imports

    def add_custom_code(self) -> list[str]:
        return [
            """
function ClerkSessionSynchronizer({ children }) {
  const { getToken, isLoaded, isSignedIn } = useAuth()
  const [ addEvents, connectErrors ] = useContext(EventLoopContext)

  useEffect(() => {
      if (isLoaded && !!addEvents) {
        if (isSignedIn) {
          getToken().then(token => {
            addEvents([Event("state.clerk_state.set_clerk_session", {token})])
          })
        } else {
          addEvents([Event("state.clerk_state.clear_clerk_session")])
        }
      }
  }, [isSignedIn])      

  return (
      <>{children}</>
  )
}
"""
        ]


class ClerkProvider(rx.Component):
    """ClerkProvider component."""

    # The React library to wrap.
    library = "@clerk/clerk-react"

    # The React component tag.
    tag = "ClerkProvider"

    publishable_key: str = None
    """
    The Clerk publishable key for your instance. This can be found in your
    Clerk Dashboard on the [API Keys][1] page.
    
    https://dashboard.clerk.com/last-active?path=api-keys
    """

    secret_key: str = None
    """
    The Clerk publishable key for your instance. This can be found in your
    Clerk Dashboard on the [API Keys][1] page.  This key should never be
    exposed to the client, and will be stripped from this tag's props before
    rendering.
    
    https://dashboard.clerk.com/last-active?path=api-keys
    """

    router_push: rx.EventHandler[lambda e: [e]]
    """A function which takes the destination path as an argument and performs a "push" navigation."""

    router_replace: rx.EventHandler[lambda e: [e]]
    """A function which takes the destination path as an argument and performs a "replace" navigation."""

    clerk_j_s_url: str = None
    """Define the URL that @clerk/clerk-react should hot-load @clerk/clerk-js from."""

    clerk_j_s_variant: str = None
    """
    If your web application only uses Control components, you can set this value to
    headless and load a minimal ClerkJS bundle for optimal page performance.
    """

    clerk_j_s_version: str = None
    """Define the npm version for @clerk/clerk-js."""

    support_email: str = None
    """
    Optional support email for display in authentication screens. Will only affect
    Clerk Components and not Account Portal pages
    """

    appearance: dict = None
    """
    Optional object to style your components. Will only affect Clerk Components
    and not Account Portal pages.
    """

    localization: dict = None
    """
    Optional object to localize your components. Will only affect Clerk Components
    and not Account Portal pages.
    """

    allowed_redirect_origins: typing.List[str] = None
    """
    Optional array of domains used to validate against the query param of an auth
    redirect. If no match is made, the redirect is considered unsafe and the default
    redirect will be used with a warning passed to the console.
    """

    sign_in_force_redirect_url: str = None
    """
    If provided, this URL will always be redirected to after the user signs in. It's
    recommended to use the environment variable instead.
    """

    sign_up_force_redirect_url: str = None
    """
    If provided, this URL will always be redirected to after the user signs up. It's
    recommended to use the environment variable instead.
    """

    sign_in_fallback_redirect_url: str = None
    """
    The fallback URL to redirect to after the user signs in, if there's no redirect_url
    in the path already. Defaults to /. It's recommended to use the environment variable
    instead.
    """

    sign_up_fallback_redirect_url: str = None
    """
    The fallback URL to redirect to after the user signs up, if there's no redirect_url
    in the path already. Defaults to /. It's recommended to use the environment variable
    instead.
    """

    is_satellite: bool = None
    """This option defines that the application is a satellite application"""

    domain: str = None
    """
    This option sets the domain of the satellite application. If your application is a
    satellite application, this option is required.
    """

    sign_in_url: str = None
    """
    This URL will be used for any redirects that might happen and needs to point to
    your primary application. This option is optional for production instances and
    required for development instances. It's recommended to use the environment variable
    instead.
    """

    telemetry: bool = None
    """Controls whether or not Clerk will collect telemetry data."""

    @classmethod
    def create(cls, *children, **props) -> 'ClerkProvider':
        # Copy secret key to ClerkState, then remove it from the props to
        # avoid passing it to the client.
        if props.get('secret_key'):
            ClerkState._secret_key = props['secret_key']
            del props['secret_key']

        # Check that at this point we have a secret key either passed as a
        # prop or set as an environment variable.
        if not ClerkState.secret_key:
            raise ValueError(
                "ClerkProvider requires a secret_key.  You can set this by passing "
                "it as a keyword argument to the clerk_provider component or by "
                "setting the CLERK_SECRET_KEY environment variable.\n\nThis can "
                "be found in your Clerk Dashboard on the API Keys page:\n"
                "https://dashboard.clerk.com/last-active?path=api-keys")

        if not props.get('publishable_key'):
            props['publishable_key'] = os.environ.get('CLERK_PUBLISHABLE_KEY')

        if not props['publishable_key']:
            raise ValueError(
                "ClerkProvider requires a publishable_key.  You can set this by passing "
                "it as a keyword argument to the clerk_provider component or by "
                "setting the CLERK_PUBLISHABLE_KEY environment variable.\n\nThis can "
                "be found in your Clerk Dashboard on the API Keys page:\n"
                "https://dashboard.clerk.com/last-active?path=api-keys")

        # Create a synchronizer and wrap it in a ClerkProvider.
        synchronizer = ClerkSessionSynchronizer.create(*children)
        clerk_provider = super().create(synchronizer, **props)

        # Wrap the ClerkProvider in a Fragment, as otherwise the outer provider component
        # seems to be removed occasionally.
        return rx.fragment(clerk_provider)


def clerk_provider(
        *children,
        publishable_key: typing.Optional[str] = None,
        secret_key: typing.Optional[str] = None,
        clerk_j_s_url: typing.Optional[str] = None,
        clerk_j_s_variant: typing.Optional[str] = None,
        clerk_j_s_version: typing.Optional[str] = None,
        support_email: typing.Optional[str] = None,
        appearance: typing.Optional[dict] = None,
        localization: typing.Optional[dict] = None,
        allowed_redirect_origins: typing.Optional[typing.List[typing.Optional[str]]] = None,
        sign_in_force_redirect_url: typing.Optional[str] = None,
        sign_up_force_redirect_url: typing.Optional[str] = None,
        sign_in_fallback_redirect_url: typing.Optional[str] = None,
        sign_up_fallback_redirect_url: typing.Optional[str] = None,
        is_satellite: typing.Optional[bool] = None,
        domain: typing.Optional[str] = None,
        sign_in_url: typing.Optional[str] = None,
        telemetry: typing.Optional[bool] = None,
) -> Component:
    """
    A component which wraps your application and provides a context for Clerk to function.

    This component must be rendered at the root of your application, before any other
    reflex_clerk components.

    Example:
        ```python
        import reflex as rx
        import reflex_clerk as clerk

        @rx.page("/")
        def index(request):
            return clerk.clerk_provider(

                # ... Child components...

                publishable_key="pk_test_123",
                secret_key="sk_test_123",
            )
        ```

    Args:
        publishable_key: The publishable key used to initialize Clerk. This can be passed either as a prop or set as the `CLERK_PUBLISHABLE_KEY` environment variable.
        secret_key: The secret key used to call out to the clerk API from the backend. This can be passed either as a prop or set as the `CLERK_SECRET_KEY` environment variable.

    Other Args:
        clerk_j_s_url: The URL that @clerk/clerk-react should hot-load @clerk/clerk-js from.
        clerk_j_s_variant: The variant of @clerk/clerk-js to load. Defaults to the latest version.
        clerk_j_s_version: The version of @clerk/clerk-js to load. Defaults to the latest version.
        support_email: Optional support email for display in authentication screens. Will only affect Clerk Components and not Account Portal pages.
        appearance: Optional object to style your components. Will only affect Clerk Components and not Account Portal pages.
        localization: Optional object to localize your components. Will only affect Clerk Components and not Account Portal pages.
        allowed_redirect_origins: Optional array of domains used to validate against the query param of an auth redirect. If no match is made, the redirect is considered unsafe and the default redirect will be used with a warning passed to the console.
        sign_in_force_redirect_url: If provided, this URL will always be redirected to after the user signs in. It's recommended to use the environment variable instead.
        sign_up_force_redirect_url: If provided, this URL will always be redirected to after the user signs up. It's recommended to use the environment variable instead.
        sign_in_fallback_redirect_url: The fallback URL to redirect to after the user signs in, if there's no redirect_url in the path already. Defaults to /. It's recommended to use the environment variable instead.
        sign_up_fallback_redirect_url: The fallback URL to redirect to after the user signs up, if there's no redirect_url in the path already. Defaults to /. It's recommended to use the environment variable instead.
        is_satellite: This option defines that the application is a satellite application.
        domain: This option sets the domain of the satellite application. If your application is a satellite application, this option is required.
        sign_in_url: This URL will be used for any redirects that might happen and needs to point to your primary application. This option is optional for production instances and required for development instances. It's recommended to use the environment variable instead.
        telemetry: Controls whether or not Clerk will collect telemetry data.

    Returns:
        ClerkProvider: A new instance of ClerkProvider.
    """
    return ClerkProvider.create(
        *children,
        publishable_key=publishable_key,
        secret_key=secret_key,
        clerk_j_s_url=clerk_j_s_url,
        clerk_j_s_variant=clerk_j_s_variant,
        clerk_j_s_version=clerk_j_s_version,
        support_email=support_email,
        appearance=appearance,
        localization=localization,
        allowed_redirect_origins=allowed_redirect_origins,
        sign_in_force_redirect_url=sign_in_force_redirect_url,
        sign_up_force_redirect_url=sign_up_force_redirect_url,
        sign_in_fallback_redirect_url=sign_in_fallback_redirect_url,
        sign_up_fallback_redirect_url=sign_up_fallback_redirect_url,
        is_satellite=is_satellite,
        domain=domain,
        sign_in_url=sign_in_url,
        telemetry=telemetry)



import requests

from .clerk_request_models import *
from .clerk_response_models import *


class ClerkAPIClient(object):
    def __init__(self, base_url: str, secret_key: str, headers: Optional[Dict[str, str]] = None):
        self.base_url = base_url
        self.secret_key = secret_key
        self.headers = headers if headers else {}
        self.headers["Authorization"] = f"Bearer {self.secret_key}"

    def _get_url(self, endpoint: str) -> str:
        return f"{self.base_url}{endpoint}"

    def _handle_response(self, response: requests.Response, response_model: Any):
        if response.status_code >= 400:
            response.raise_for_status()
        try:
            data = response.json()
            return response_model.parse_obj(data)
        except ValueError:
            raise ValueError(f"Failed to parse response '{response.text}' as {response_model.__name__}")

    def get_public_interstitial(self, params: Optional[GetPublicInterstitialParams] = None) -> InterstitialResponse:
        url = self._get_url("/public/interstitial")
        response = requests.get(url, headers=self.headers, params=params.dict() if params else None)
        return self._handle_response(response, InterstitialResponse)

    def get_jwks(self) -> JWKSResponse:
        url = self._get_url("/jwks")
        response = requests.get(url, headers=self.headers)
        return self._handle_response(response, JWKSResponse)

    def get_client_list(self, params: Optional[GetClientListParams] = None) -> ClientListResponse:
        url = self._get_url("/clients")
        response = requests.get(url, headers=self.headers, params=params.dict() if params else None)
        return self._handle_response(response, ClientListResponse)

    def verify_client(self, data: VerifyClientRequest) -> ClientResponse:
        url = self._get_url("/clients/verify")
        response = requests.post(url, headers=self.headers, json=data.dict())
        return self._handle_response(response, ClientResponse)

    def get_client(self, client_id: str) -> ClientResponse:
        url = self._get_url(f"/clients/{client_id}")
        response = requests.get(url, headers=self.headers)
        return self._handle_response(response, ClientResponse)

    def create_email_address(self, data: CreateEmailAddressRequest) -> EmailAddressResponse:
        url = self._get_url("/email_addresses")
        response = requests.post(url, headers=self.headers, json=data.dict())
        return self._handle_response(response, EmailAddressResponse)

    def get_email_address(self, email_address_id: str) -> EmailAddressResponse:
        url = self._get_url(f"/email_addresses/{email_address_id}")
        response = requests.get(url, headers=self.headers)
        return self._handle_response(response, EmailAddressResponse)

    def delete_email_address(self, email_address_id: str) -> DeletedObjectResponse:
        url = self._get_url(f"/email_addresses/{email_address_id}")
        response = requests.delete(url, headers=self.headers)
        return self._handle_response(response, DeletedObjectResponse)

    def update_email_address(self, email_address_id: str, data: UpdateEmailAddressRequest) -> EmailAddressResponse:
        url = self._get_url(f"/email_addresses/{email_address_id}")
        response = requests.patch(url, headers=self.headers, json=data.dict())
        return self._handle_response(response, EmailAddressResponse)

    def create_phone_number(self, data: CreatePhoneNumberRequest) -> PhoneNumberResponse:
        url = self._get_url("/phone_numbers")
        response = requests.post(url, headers=self.headers, json=data.dict())
        return self._handle_response(response, PhoneNumberResponse)

    def get_phone_number(self, phone_number_id: str) -> PhoneNumberResponse:
        url = self._get_url(f"/phone_numbers/{phone_number_id}")
        response = requests.get(url, headers=self.headers)
        return self._handle_response(response, PhoneNumberResponse)

    def delete_phone_number(self, phone_number_id: str) -> DeletedObjectResponse:
        url = self._get_url(f"/phone_numbers/{phone_number_id}")
        response = requests.delete(url, headers=self.headers)
        return self._handle_response(response, DeletedObjectResponse)

    def update_phone_number(self, phone_number_id: str, data: UpdatePhoneNumberRequest) -> PhoneNumberResponse:
        url = self._get_url(f"/phone_numbers/{phone_number_id}")
        response = requests.patch(url, headers=self.headers, json=data.dict())
        return self._handle_response(response, PhoneNumberResponse)

    def get_session_list(self, params: Optional[GetSessionListParams] = None) -> SessionListResponse:
        url = self._get_url("/sessions")
        response = requests.get(url, headers=self.headers, params=params.dict() if params else None)
        return self._handle_response(response, SessionListResponse)

    def get_session(self, session_id: str) -> SessionResponse:
        url = self._get_url(f"/sessions/{session_id}")
        response = requests.get(url, headers=self.headers)
        return self._handle_response(response, SessionResponse)

    def revoke_session(self, session_id: str) -> SessionResponse:
        url = self._get_url(f"/sessions/{session_id}/revoke")
        response = requests.post(url, headers=self.headers)
        return self._handle_response(response, SessionResponse)

    def verify_session(self, session_id: str, data: VerifySessionRequest) -> SessionResponse:
        url = self._get_url(f"/sessions/{session_id}/verify")
        response = requests.post(url, headers=self.headers, json=data.dict())
        return self._handle_response(response, SessionResponse)

    def create_session_token_from_template(self, session_id: str, template_name: str) -> Dict[str, str]:
        url = self._get_url(f"/sessions/{session_id}/tokens/{template_name}")
        response = requests.post(url, headers=self.headers)
        return self._handle_response(response, Dict[str, str])

    def get_template_list(self, template_type: str) -> TemplateListResponse:
        url = self._get_url(f"/templates/{template_type}")
        response = requests.get(url, headers=self.headers)
        return self._handle_response(response, TemplateListResponse)

    def get_template(self, template_type: str, slug: str) -> TemplateResponse:
        url = self._get_url(f"/templates/{template_type}/{slug}")
        response = requests.get(url, headers=self.headers)
        return self._handle_response(response, TemplateResponse)

    def upsert_template(self, template_type: str, slug: str, data: UpsertTemplateRequest) -> TemplateResponse:
        url = self._get_url(f"/templates/{template_type}/{slug}")
        response = requests.put(url, headers=self.headers, json=data.dict())
        return self._handle_response(response, TemplateResponse)

    def revert_template(self, template_type: str, slug: str) -> TemplateResponse:
        url = self._get_url(f"/templates/{template_type}/{slug}/revert")
        response = requests.post(url, headers=self.headers)
        return self._handle_response(response, TemplateResponse)

    def preview_template(self, template_type: str, slug: str, data: PreviewTemplateRequest) -> Dict[str, Any]:
        url = self._get_url(f"/templates/{template_type}/{slug}/preview")
        response = requests.post(url, headers=self.headers, json=data.dict())
        return self._handle_response(response, Dict[str, Any])

    def toggle_template_delivery(self, template_type: str, slug: str,
                                 data: ToggleTemplateDeliveryRequest) -> TemplateResponse:
        url = self._get_url(f"/templates/{template_type}/{slug}/toggle_delivery")
        response = requests.post(url, headers=self.headers, json=data.dict())
        return self._handle_response(response, TemplateResponse)

    def get_user_list(self, params: Optional[GetUserListParams] = None) -> UserListResponse:
        url = self._get_url("/users")
        response = requests.get(url, headers=self.headers, params=params.dict() if params else None)
        return self._handle_response(response, UserListResponse)

    def create_user(self, data: CreateUserRequest) -> User:
        url = self._get_url("/users")
        response = requests.post(url, headers=self.headers, json=data.dict())
        return self._handle_response(response, User)

    def get_users_count(self, params: Optional[GetUsersCountParams] = None) -> UserCountResponse:
        url = self._get_url("/users/count")
        response = requests.get(url, headers=self.headers, params=params.dict() if params else None)
        return self._handle_response(response, UserCountResponse)

    def get_user(self, user_id: str) -> User:
        url = self._get_url(f"/users/{user_id}")
        response = requests.get(url, headers=self.headers)
        return self._handle_response(response, User)

    def update_user(self, user_id: str, data: UpdateUserRequest) -> User:
        url = self._get_url(f"/users/{user_id}")
        response = requests.patch(url, headers=self.headers, json=data.dict())
        return self._handle_response(response, User)


BASE_URL = "https://api.clerk.com/v1"


def get_client(secret_key) -> ClerkAPIClient:
    """
    Returns an instance of the ClerkAPIClient using the provided secret key.

    :param secret_key: The secret key used to authenticate the client.
    :type secret_key: str
    :return: An instance of the ClerkAPIClient.
    :rtype: ClerkAPIClient
    """
    return ClerkAPIClient(BASE_URL, secret_key)
